;;; ivan-emms.el --- emacs multimedia system

;;; Commentary:

;;; THANKS:

;;; BUGS:

;;; INSTALLATION:

;;; Code:

(require 'ivan-face)

(eval-when-compile
  (setq load-path (cons "../emms" load-path)))

(defvar ivan-emms-shuffle nil
  "t when playlist is shuffled.")

(defvar ivan-emms-dir "~/music"
  "Directory where music is stored.")

(defvar ivan-emms-last (concat ivan-var-tmp-dir "emms-last.el")
  "Cache of last played albums")

(defun ivan-emms-add-dired ()
  (interactive)
  (ivan-emms-init)
  (emms-add-dired))

(defun ivan-emms-init ()
  (if (not (fboundp 'emms-playlist-mode-go))
      (progn
        (require 'emms)
        (require 'emms-playlist-mode)
        (require 'emms-browser)
        (require 'emms-setup)
        (require 'emms-tag-editor)
        (require 'url-util)
        (require 'emms-playlist-mode)
        (emms-minimalistic)
        (setq emms-playlist-default-major-mode 'emms-playlist-mode))))

(defun ivan-emms-message-info ()
  (interactive)
  (let ((track (emms-playlist-current-selected-track)))
    (emms-info-really-initialize-track track)
    (message (emms-info-track-description track))))

(add-hook 'emms-playlist-selection-changed-hook
          'ivan-emms-message-info)

(defun ivan-emms-next-album ()
  "Play next album in track list."
  (interactive)
  (let ((dir (ivan-emms-track-dir)))
    (while (string= dir (ivan-emms-track-dir))
      (emms-next))))

(defun ivan-emms-track-info ()
  "Return string containing artist and track name"
  (when (and (fboundp 'emms-playlist-current-selected-track)
             (fboundp 'emms-info-track-description))
    (let ((selected-track (emms-playlist-current-selected-track)))
      (when selected-track
        (let ((track (emms-info-track-description selected-track)))
          (concat (nth 2 (nreverse (split-string track "/")))
                  " - "
                  (replace-regexp-in-string
                   ".*[0-9][0-9] - " ""
                   (file-name-sans-extension
                    (file-name-nondirectory track)))))))))

(defun ivan-emms-track-dir ()
  "Get directory of current playing track."
  (file-name-directory
   (emms-info-track-description
    (emms-playlist-current-selected-track))))

(defun ivan-emms-play-dired ()
  (interactive)
  (ivan-emms-init)
  (emms-play-dired))

(defun ivan-emms-playlist-mode-go ()
  (require 'emms-playlist-mode)
  (setq emms-playlist-default-major-mode 'emms-playlist-mode)
  (emms-playlist-mode-go))

;;; random album picker
(defun ivan-emms-get-dir (path)
  (let ((dir (cddr (directory-files path)))
        (ret nil)
        file)
    (while dir
      (setq file (concat path "/" (car dir)))
      (if (file-directory-p file)
          (setq ret (cons file ret)))
      (setq dir (cdr dir)))
    ret))

(defun ivan-emms-list-subdir (path)
  (let ((dir (ivan-emms-get-dir path))
        (ret nil))
    (while dir
      (if (string-match "music" path)
          (setq ret (append ret (ivan-emms-get-dir (car dir))))
        (setq ret (append ret dir)))
      (setq dir (cdr dir)))
    ret))

(defun ivan-emms-pick-album (dir length with-cache)
  "Return a LENGTH albums in a directory DIR.
If WITH-CACHE is non-nil exclude albums that are in cache."
  (let* ((index 0)
         (ret (make-vector length nil))
         (album (ivan-emms-list-subdir
                 (concat ivan-emms-dir dir)))
         (album-length (length album))
         ring)
    (when with-cache
      (setq ring (ivan-emms-last-load)))
    (while (< index length)
      (aset ret index
            (nth (if with-cache
                     (let* ((r (random album-length))
                            (album (nth r album)))
                       (while (ring-member ring r)
                         (setq r (random album-length)
                               album (nth r album)))
                       (ring-insert ring album)
                       r)
                   (random album-length))
                 album))
      (setq index (1+ index)))
    (when with-cache
      (ivan-emms-last-save ring))
    ret))

(defun ivan-emms-play-list (playlist)
  (let ((length (length playlist))
        (index 0))
    (emms-playlist-current-clear)
    ;; add one various song to top of playlist
    (emms-add-file (ivan-emms-random-file
                    (ivan-emms-just-file
                     (directory-files
                      (concat ivan-emms-dir "music/Various") t))))
    (while (< index length)
      (emms-add-directory (aref playlist index))
      (setq index (1+ index)))))

(defun ivan-emms-random-album ()
  "Play albums from my library at random"
  (interactive)
  (ivan-emms-init)
  (random t)
  (let ((playlist (vconcat (ivan-emms-pick-album "music" 15 t)
                           (ivan-emms-pick-album "fresh" 15 nil))))
    (emms-shuffle-vector playlist)
    (ivan-emms-play-list playlist)
    (emms-start)))

(defun ivan-emms-just-file (list)
  "Remove directories from LIST."
  (when list
    (if (file-regular-p (car list))
        (cons (car list) (ivan-emms-just-file (cdr list)))
      (ivan-emms-just-file (cdr list)))))

(defun ivan-emms-random-file (file)
  "Given a list of FILE, pick one at random"
  (nth (random (length file)) file))

(defun ivan-emms-last-save (ring)
  "Save last played directories."
  (with-temp-file ivan-emms-last
    (insert "(setq emms-last '")
    (prin1 (ring-elements ring) (current-buffer))
    (insert ")")))

(defun ivan-emms-last-load ()
  "Return ring of last played directories."
  (if (load ivan-emms-last t)
      (ring-convert-sequence-to-ring emms-last)
    (let* ((length 50)
           (r (make-ring length))
           (i 0))
      (while (< i length)
        (ring-insert r "")
        (setq i (1+ i)))
      r)))

(setq
 emms-info-asynchronously nil
 emms-player-list (list 'emms-player-mplayer-playlist 'emms-player-mplayer)
 emms-playing-time-display-format "")

(ivan-face
 emms-browser-year/genre-face yellow
 emms-browser-artist-face yellow
 emms-browser-album-face yellow
 emms-browser-track-face normal
 emms-playlist-track-face normal
 emms-playlist-selected-face yellow)

(provide 'ivan-emms)

;; Local Variables:
;; compile-command: "make"
;; End:

;; Copyright (C) 2009 Ivan Kanis
;; Author: Ivan Kanis
;;
;; This program is free software ; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation ; either version 2 of the License, or
;; (at your option) any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY ; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program ; if not, write to the Free Software
;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
